Attribute VB_Name = "mBitmap"
Option Explicit

'Types
    Type tRect
        'Coordinates
        X As Long
        Y As Long
        
        X2 As Long
        Y2 As Long
    End Type
    
    Type tBitmap
        'System
        FileName As String
        
        DC As Long
        Mask As Long
        
        'Size
        w As Long
        h As Long
    End Type
    
'Declares
    Public Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
    Public Declare Function FillRect Lib "user32" (ByVal hdc As Long, lpRect As tRect, ByVal hBrush As Long) As Long
    
    Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
    Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
    
    Public Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Long, ByVal hdc As Long) As Long
    
    Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
    Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
    
    Public Declare Function SetPixel Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long
    Public Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long) As Long

'Variables
    Public BitmapCount As Long
    Public Bitmap() As tBitmap
    
    Private MainWindow As Form
Sub GenerateMask(iBitmap As Long)
    Dim X As Long
    Dim Y As Long
    
    'Create mask
    With Bitmap(iBitmap)
        'Allocate memory
        ReleaseDC MainWindow.hwnd, .Mask
        .Mask = CreateDC(.w, .h)
        
        'Generate mask
        For Y = 0 To .h
            For X = 0 To .w
                SetPixel .Mask, X, Y, IIf(GetPixel(.DC, X, Y) = 16777215, 0, 16777215)
            Next
        Next
    End With
End Sub

Sub Initialize(iWindow As Form)
    'Variables
    BitmapCount = -1
    
    Set MainWindow = iWindow
    MainWindow.ScaleMode = vbPixels
    
    Reset
End Sub
Function LoadDC(iFileName As String, iBitmap As tBitmap) As Long
    Dim DC As Long
    Dim Temp As IPictureDisp
    
    'Create compatible DC
    DC = CreateCompatibleDC(MainWindow.hdc)
    
    If DC < 1 Then
        'Error: Can't create compatible DC
        LoadDC = -1
        
        Exit Function
    End If
    
    'Load bitmap
    Set Temp = LoadPicture(iFileName)
    SelectObject DC, Temp
    
    'Apply values
    With iBitmap
        .FileName = LCase(iFileName)
        
        .DC = DC
        LoadDC = DC
        
        .w = MainWindow.ScaleX(Temp.Width)
        .h = MainWindow.ScaleY(Temp.Height)
    End With
    
    'Release memory
    DeleteObject Temp
    Set Temp = Nothing
End Function

Function CreateDC(iW As Long, iH As Long, Optional iClear As Boolean = False) As Long
    Dim Temp As Long
    Dim Rect As tRect
    
    'Allocate memory
    CreateDC = CreateCompatibleDC(MainWindow.hdc)
    Temp = CreateCompatibleBitmap(MainWindow.hdc, iW, iH)
    
    'Get DC
    SelectObject CreateDC, Temp
    DeleteObject Temp
    
    If iClear Then
        'Clear DC
        Rect.X2 = iW
        Rect.Y2 = iH
        
        FillRect CreateDC, Rect, 0
    End If
End Function
Function LoadBitmap(iFileName As String, Optional iMasked As Boolean = False) As Long
    Dim A As Long

    If Dir(iFileName) = "" Then
        'Error: File not found
        LoadBitmap = -1
        MsgBox iFileName & " not found.", vbCritical, "Error"
        
        Exit Function
    End If
    
    'Check if already loaded
    For A = 0 To BitmapCount
        If Bitmap(A).FileName = LCase(iFileName) Then
            'Bitmap already loaded
            LoadBitmap = A
            
            Exit Function
        End If
    Next
    
    'Allocate memory
    BitmapCount = BitmapCount + 1
    ReDim Preserve Bitmap(BitmapCount)
    
    'Load file
    LoadDC iFileName, Bitmap(BitmapCount)
    
    'Masking
    If iMasked Then
        GenerateMask BitmapCount
    Else
        Bitmap(BitmapCount).Mask = -1
    End If
    
    'Return index
    If Bitmap(BitmapCount).DC > -1 Then
        LoadBitmap = BitmapCount
    Else
        LoadBitmap = -1
    End If
End Function
Sub Reset()
    Dim A As Long
    
    'Release DCs
    For A = 0 To BitmapCount
        ReleaseDC MainWindow.hwnd, Bitmap(A).DC
        ReleaseDC MainWindow.hwnd, Bitmap(A).Mask
    Next
    
    'Release memory
    BitmapCount = -1
    Erase Bitmap
End Sub
Sub ShutDown()
    'Release memory
    Reset
End Sub

Sub UnloadBitmap(Index As Long)
    'Release DCs
    ReleaseDC MainWindow.hwnd, Bitmap(Index).DC
    ReleaseDC MainWindow.hwnd, Bitmap(Index).Mask
    
    Bitmap(Index).FileName = ""
End Sub
