Attribute VB_Name = "mCore"
Option Explicit

'Consts
    Public Const DebugMode As Boolean = True
    Public Const NL As String = vbNewLine

'Declares
    Public Declare Function GetTickCount Lib "kernel32" () As Long

'Variables
    Public Version As String
    Public Path As String
    
    Public Settings As New cSettings
    
    Public Active As Boolean
    Public MaxFPS As Long
    
    Public BackBitmap As Long
    Public MouseBitmap As Long
    Public CursorBitmap As Long
    
    Public FrontDC As Long
    
    Public ScreenW As Long
    Public ScreenH As Long
    
    Public Points As Single
    Public Missed As Long
    Public Level As Long
    
    Public BackPos As Single
    Public BackSpeed As Single
    
    Public GameOver As Boolean
    
    Public Credits() As String
    Public CreditsCount As Long
    
    Public NoSound As Boolean
    
    Public HiScore As Long
    Public HiCombo As Long
Sub AddCredits()
    Static Temp As Long
    Static ActCredits As Long
    
    If Temp < GetTickCount Then
        'Add text
        mText.Add Credits(ActCredits), 80 + 50 * Rnd, ScreenW * Rnd, ScreenH * Rnd, 16777215 * Rnd, 1, False
        ActCredits = ActCredits + 1
        If ActCredits > CreditsCount Then: ActCredits = 0
        
        Temp = GetTickCount + 1000
    End If
End Sub
Sub AddPoints(iPoints As Single, iX As Single, iY As Single)
On Error Resume Next
    Points = Points + iPoints
    
    If iPoints > 0 Then
        'Increase level.. level 2 would be at 400 points
        If 1 + Int(Points / (Level * 100)) > Level Then: SetLevel 1 + Int(Points / (Level * 100))
        
        mText.Add CStr(Int(iPoints)), 12 + IIf(Points > 40, 40, Points), iX, iY, RGB((iPoints - 10) * 12, 0, 0), -BackSpeed
        
    Else
        mText.Add CStr(Int(iPoints)), 12 + IIf(-iPoints > 40, 40, -iPoints), iX, iY, RGB(0, 0, (-iPoints + 64) * 20), -1
    End If
    
    If Points < 0 Then
        If Not GameOver Then
            GameOver = True
            
            mText.Add "GAME OVER", 48, ScreenW / 2, ScreenH / 2 - 50, 255, 0, True
            mText.Add "Level: " & CStr(Level), 24, ScreenW / 2, ScreenH / 2 + 20, 0, 0, True
            mText.Add "Hiscore: " & CStr(HiScore), 24, ScreenW / 2, ScreenH / 2 + 60, 0, 0, True
            mText.Add "Combos: " & CStr(HiCombo), 24, ScreenW / 2, ScreenH / 2 + 100, 0, 0, True
            
            mSound.PlaySound Path & "gameover.wav", False, 3
            
            frmMain.lblStart.Caption = "Go!"
        End If
        
        Points = 0
        Level = 1
        Missed = 0
    End If
    
    If Points > HiScore Then: HiScore = Points
End Sub
Sub DrawCursor()
    'Draw cursor
    BitBlt FrontDC, MouseX - 32, MouseY - 32, 64, 64, Bitmap(CursorBitmap).DC, 64, 0, vbSrcPaint
    BitBlt FrontDC, MouseX - 32, MouseY - 32, 64, 64, Bitmap(CursorBitmap).DC, 0, 0, vbSrcAnd
End Sub

Sub Initialize()
    'General
    Randomize
    
    MaxFPS = 30
    MinMouseCount = 5
    
    'Initialize all
    mBitmap.Initialize frmMain
    mMouse.Initialize
    mText.Initialize
    mSound.Initialize
End Sub

Function CreatePath(iPath As String) As String
    Dim A As Long
    Dim Directory() As String
    Dim Temp As String
    
    'Variables
    Directory = Split(iPath, "\")
    Temp = Directory(0) & "\"
    
    'Check each parent directory
    For A = 1 To UBound(Directory, 1)
        If Not Directory(A) = "" Then
            Temp = Temp & Directory(A) & "\"
            
            If Dir(Temp, vbDirectory) = "" Then: MkDir Temp
        End If
    Next
    
    If Not Dir(Temp, vbDirectory) = "" Then
        'Return created directory
        CreatePath = Temp
    Else
        'Error: Can't create path
        CreatePath = ""
    End If
End Function
Function Exists(iFileName As String) As Boolean
    Exists = Not (Dir(iFileName, vbArchive Or vbHidden Or vbNormal Or vbReadOnly Or vbSystem) = "")
End Function
Sub SetLevel(iLevel As Long)
    Dim A As Long
    
    Level = iLevel
    
    For A = 0 To 10
        mText.Add "Level " & CStr(Level), 10, ScreenW * Rnd, ScreenH * Rnd, 16777215, -1, False
    Next
    
    mSound.PlaySound Path & "nextlevel.wav", , 0
End Sub

Sub ShutDown()
    'Release memory
    Settings.SaveData
    mBitmap.ShutDown
    mMouse.Reset
    mText.ShutDown
    mSound.ShutDown
    
    End
End Sub

Sub Reset()
    'Reset all
    mBitmap.Reset
    mMouse.Reset
    mText.Reset
End Sub
Sub Main()
    'System
    Version = App.Major & "." & App.Minor & "." & App.Revision
    
    Path = CreatePath(App.Path)
    
    'Initialize
    Load frmMain
    InstallWavMix
    Initialize
    
    'Credits
    CreditsCount = 2
    ReDim Credits(CreditsCount)
    Credits(0) = "Fox McCloud"
    Credits(1) = "Mouse Attack"
    Credits(2) = "2000"
    
    LoadBitmaps
    
    'Load settings
    Settings.LoadData Path & "Settings.ini"
    
    'Show
    frmMain.Show
    frmMain.RunGame
End Sub
Sub MainLoop()
    Dim Temp As Long
    
    While Active
        Temp = GetTickCount
        
        If GameOver Then: AddCredits
        
        mMouse.Move
        mMouse.Animate
        mMouse.Draw
        
        mText.Move
        mText.CleanUp
        mText.Draw
        
        DrawCursor
        
        UpdateGUI
            
        'Timing
        frmMain.Front.Refresh
        
        Do: DoEvents: Loop While Temp + (1000 / MaxFPS) > GetTickCount
    Wend
End Sub
Sub LoadBitmaps()
    'Load bitmaps
    BackBitmap = LoadBitmap(Path & "Back.jpg", False)
    MouseBitmap = LoadBitmap(Path & "Mouse.bmp", True)
    CursorBitmap = LoadBitmap(Path & "Cross.bmp", True)
End Sub

Sub UpdateGUI()
    Dim Temp As String
    
    'Points
    Temp = Int(Points)
    If frmMain.lblPoints.Caption <> Temp Then: frmMain.lblPoints.Caption = Temp
    
    'Missed
    Temp = CStr(Missed)
    If frmMain.lblMissed.Caption <> Temp Then: frmMain.lblMissed.Caption = Temp
    
    'Level
    Temp = CStr(Level)
    If frmMain.lblLevel.Caption <> Temp Then: frmMain.lblLevel.Caption = Temp
    
    'Add mouse if neccessary
    If MouseCount - MinMouseCount < Level Then: mMouse.Add
End Sub


