Attribute VB_Name = "mSound"
Option Explicit

'Consts
    Global Const WaveMix_MaxChannels = 4

'Types
    Private Type tChannelInfo
        Loops As Long
        WaveFile As String
    End Type
    
    Private Type WaveMixInfo
        wSize As Integer
        dwFormats As Long
        szDate(12) As String
        bVersionMajor As String * 1
        bVersionMinor As String * 1
    End Type
    
    Private Type MixConfig
        wSize As Integer
        dwFlagsLo As Integer
        dwFlagsHi As Integer
        wChannels As Integer
        wSamplingRate As Integer
    End Type
    
    Private Type MixPlayParams
        wSize As Integer
        hMixSessionLo As Integer
        hMixSessionHi As Integer
        iChannelLo As Integer
        iChannelHi As Integer
        lpMixWaveLo As Integer
        lpMixWaveHi As Integer
        hWndNotifyLo As Integer
        hWndNotifyHi As Integer
        dwFlagsLo As Integer
        dwFlagsHi As Integer
        wLoops As Integer
    End Type

'Declares
    Private Declare Sub WaveMixPump Lib "wavmix32.dll" ()
    Private Declare Function WaveMixInit Lib "wavmix32.dll" () As Long
    Private Declare Function WaveMixConfigureInit Lib "wavmix32.dll" (lpConfig As MixConfig) As Long
    Private Declare Function WaveMixActivate Lib "wavmix32.dll" (ByVal hMixSession As Long, ByVal fActivate As Integer) As Long
    Private Declare Function WaveMixOpenWave Lib "wavmix32.dll" (ByVal hMixSession As Long, szWaveFilename As Any, ByVal hInst As Long, ByVal dwFlags As Long) As Long
    Private Declare Function WaveMixOpenChannel Lib "wavmix32.dll" (ByVal hMixSession As Long, ByVal iChannel As Long, ByVal dwFlags As Long) As Long
    Private Declare Function WaveMixPlay Lib "wavmix32.dll" (lpMixPlayParams As Any) As Integer
    Private Declare Function WaveMixFlushChannel Lib "wavmix32.dll" (ByVal hMixSession As Long, ByVal iChannel As Integer, ByVal dwFlags As Long) As Integer
    Private Declare Function WaveMixCloseChannel Lib "wavmix32.dll" (ByVal hMixSession As Long, ByVal iChannel As Integer, ByVal dwFlags As Long) As Integer
    Private Declare Function WaveMixFreeWave Lib "wavmix32.dll" (ByVal hMixSession As Long, ByVal lpMixWave As Long) As Integer
    Private Declare Function WaveMixCloseSession Lib "wavmix32.dll" (ByVal hMixSession As Long) As Integer
    Private Declare Function WaveMixGetInfo Lib "wavmix32.dll" (lpWaveMixInfo As WaveMixInfo) As Integer
    Private Declare Function GetSystemDirectory Lib "kernel32" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long

'Variables
    Dim ChannelCount As Long
    Dim lpWaveMix() As Long
    
    Dim hWaveMix As Long
    Dim WaveHandle As Long
    Dim WavMix_Quiet As Integer
Sub InstallWavMix()
    Dim Temp As String
    Dim Length As Long
    Dim SysDir As String
    
    'Get system directory
    Temp = Space(255)
    Length = GetSystemDirectory(Temp, 255)
    SysDir = Left(Temp, Length) & "\"
    
    If Dir(SysDir & "wavmix32.dll") = "" Then
        'WaveMix dll not found in system directory
        If MsgBox("wavmix32.dll was not found in your system directory.u" & NL & NL & "Install it now?", vbQuestion Or vbYesNo, "WaveMix dll not found") = vbYes Then
            If Dir(Path & "wavmix32.dl_") = "" Then
                'File not found
                MsgBox "wavmix32.dl_ was not found in current directory.", vbCritical, "Error installing WaveMix"
            Else
                'Install dll
                FileCopy Path & "wavmix32.dl_", SysDir & "wavmix32.dll"
            End If
            
        Else
            MsgBox "Cannot play sounds.", vbInformation, "Warning"
            NoSound = True
        End If
    End If
    
    'Check application directory
    If Not Dir(Path & "wavmix32.dll") = "" Then
        If MsgBox("Found wavmix32.dll in current directory." & NL & NL & "Do you want to rename it to wavmix32.dl_ ?", vbQuestion Or vbYesNo, "Warning") = vbYes Then
            'Found WaveMix dll
            Name Path & "wavmix32.dll" As Path & "wavmix32.dl_"
        Else
            MsgBox "The program may not run properly.", vbInformation, "Warning"
        End If
    End If
End Sub
Private Function HiWord(ByVal iValue As Long) As Integer
    iValue = iValue \ &H10000
    
    HiWord = Val("&H" & Hex$(iValue))
End Function

Private Function LoWord(ByVal iValue As Long) As Integer
    iValue = iValue And &HFFFF&
    
    LoWord = Val("&H" & Hex$(iValue))
End Function


Function AddSoundFile(iFileName As String) As Integer
    Dim Temp As Long
    
    'Check if available
    If WavMix_Quiet Then: Exit Function
    If WaveHandle + 1 = WaveMix_MaxChannels Then: Exit Function
    
    'Resize memory
    ReDim Preserve lpWaveMix(WaveHandle)
    WaveHandle = WaveHandle + 1
    
    'Assign to the channel
    lpWaveMix(WaveHandle) = WaveMixOpenWave(hWaveMix, ByVal iFileName, 0, 0)
    Temp = WaveMixOpenChannel(hWaveMix, WaveHandle, 0)
    
    'Return handle
    AddSoundFile = WaveHandle
End Function

Sub SetChannelFile(iFileName As String, iChannel As Long)
    Dim Temp As Long
    
    If WavMix_Quiet Then: Exit Sub
    
    'Resize memory
    If iChannel > UBound(lpWaveMix) Then
        ReDim Preserve lpWaveMix(iChannel)
        WaveHandle = iChannel
    End If
    
    'Free channel
    If lpWaveMix(iChannel) <> 0 Then
        StopChannel iChannel
        Temp = WaveMixFreeWave(hWaveMix, lpWaveMix(iChannel))
    End If
    
    'Open channel
    lpWaveMix(iChannel) = WaveMixOpenWave(hWaveMix, ByVal iFileName, 0, 0)
    Temp = WaveMixOpenChannel(hWaveMix, iChannel, 0)
End Sub


Sub ShutDown()
    Dim Temp As Long
    Dim A As Long
    Dim rc As Integer
    
    If WavMix_Quiet Then: Exit Sub
    
    If hWaveMix <> 0 Then
        'Close channels
        For A = 0 To UBound(lpWaveMix)
            If lpWaveMix(A) <> 0 Then
                StopChannel A
                rc = WaveMixFreeWave(hWaveMix, lpWaveMix(A))
            End If
        Next
        
        'Close session
        Temp = WaveMixCloseSession(hWaveMix)
        hWaveMix = 0
    End If
End Sub

Function Initialize() As Integer
    Dim Temp As Long
    Dim Config As MixConfig

    If WavMix_Quiet Then: Exit Function
    
    'Resize memory
    ChannelCount = -1
    WaveHandle = 0
    ReDim lpWaveMix(0)
    
    'Configuration
    Config.wSize = Len(Config)
    Config.dwFlagsHi = 1
    Config.dwFlagsLo = 0
    Config.wChannels = 2
    
    'Apply configuration
    hWaveMix = WaveMixConfigureInit(Config)
    Temp = WaveMixActivate(hWaveMix, True)

    'Return if succeeded
    If Temp = 0 Then
        Initialize = True
        
    Else
        'Close wavmix
        WaveMixCloseSession hWaveMix
        hWaveMix = 0
    End If
End Function

Sub StopChannel(ByVal iChannel As Long)
    Dim Temp As Integer
    
    'Check if available
    If WavMix_Quiet Then: Exit Sub
    If hWaveMix = 0 Then: Exit Sub
    
    'Stop playing the channel
    Temp = WaveMixFlushChannel(hWaveMix, iChannel, 0)
End Sub

Sub Activate(iActivate As Long)
    Dim Temp As Integer
    
    'Check if loaded
    If WavMix_Quiet Then: Exit Sub
    If (hWaveMix = 0) Then: Exit Sub
    
    'Activate
    Temp = WaveMixActivate(hWaveMix, iActivate)
End Sub

Sub PlaySound(iFileName As String, Optional iLoop As Boolean = False, Optional iChannel As Long = 1)
    'Play sound
    If Not NoSound Then
        SetChannelFile iFileName, iChannel
        
        PlayChannel iChannel, IIf(iLoop = True, 1, 0)
    End If
End Sub
Sub StopSound(iChannel As Long)
    PlaySound vbNullString, False, iChannel
End Sub
Sub PlayChannel(iChannel As Long, iLoop As Long)
    Dim Params As MixPlayParams
    Dim Temp As Long
    
    'Check if available
    If WavMix_Quiet Then: Exit Sub
    If iChannel > UBound(lpWaveMix) Then: Exit Sub
    If hWaveMix = 0 Then: Exit Sub
    
    'Create channel
    Params.wSize = Len(Params)
    Params.hMixSessionLo = LoWord(hWaveMix)
    Params.hMixSessionHi = HiWord(hWaveMix)
    Params.iChannelLo = LoWord(iChannel)
    Params.iChannelHi = HiWord(iChannel)
    Params.lpMixWaveLo = LoWord(lpWaveMix(iChannel))
    Params.lpMixWaveHi = HiWord(lpWaveMix(iChannel))
    Params.hWndNotifyLo = 0
    Params.hWndNotifyHi = 0
    Params.dwFlagsHi = 5
    Params.dwFlagsLo = 0
    Params.wLoops = iLoop
    
    'Play channel
    Temp = WaveMixPlay(Params)
End Sub


